@@ -1,4 +1,4 @@
-/* $Id: cut.c 4911 2014-05-26 07:53:20Z bens $ */
+/* $Id: cut.c 5041 2014-06-30 20:39:27Z bens $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
@@ -37,6 +37,12 @@ void cutbuffer_reset(void)
     keep_cutbuffer = FALSE;
 }
 
+/* Return the status of cutbuffer preservation. */
+inline bool keeping_cutbuffer(void)
+{
+    return keep_cutbuffer;
+}
+
 /* If we aren't on the last line of the file, move all the text of the
  * current line, plus the newline at the end, into the cutbuffer.  If we
  * are, move all of the text of the current line into the cutbuffer.  In
@@ -114,11 +120,11 @@ void cut_to_eof(void)
 
 /* Move text from the current filestruct into the cutbuffer.  If
  * copy_text is TRUE, copy the text back into the filestruct afterward.
- * If cut_till_end is TRUE, move all text from the current cursor
+ * If cut_till_eof is TRUE, move all text from the current cursor
  * position to the end of the file into the cutbuffer. */
 void do_cut_text(
 #ifndef NANO_TINY
-	bool copy_text, bool cut_till_end, bool undoing
+	bool copy_text, bool cut_till_eof, bool undoing
 #else
 	void
 #endif
@@ -167,8 +173,8 @@ void do_cut_text(
     keep_cutbuffer = TRUE;
 
 #ifndef NANO_TINY
-    if (cut_till_end) {
-	/* If cut_till_end is TRUE, move all text up to the end of the
+    if (cut_till_eof) {
+	/* If cut_till_eof is TRUE, move all text up to the end of the
 	 * file into the cutbuffer. */
 	cut_to_eof();
     } else if (openfile->mark_set) {
@@ -209,7 +215,7 @@ void do_cut_text(
 	if (!old_no_newlines)
 	    UNSET(NO_NEWLINES);
     } else if (!undoing)
-	update_undo(cut_till_end ? CUT_EOF : CUT);
+	update_undo(cut_till_eof ? CUT_EOF : CUT);
 
     /* Leave the text in the cutbuffer, and mark the file as
      * modified. */
@@ -245,14 +251,24 @@ void do_cut_text_void(void)
 
 #ifndef NANO_TINY
 /* Move text from the current filestruct into the cutbuffer, and copy it
- * back into the filestruct afterward. */
+ * back into the filestruct afterward.  If the mark is set or the cursor
+ * was moved, blow away previous contents of the cutbuffer. */
 void do_copy_text(void)
 {
+    static struct filestruct *next_contiguous_line = NULL;
+    bool mark_set = openfile->mark_set;
+
+    if (mark_set || openfile->current != next_contiguous_line)
+	cutbuffer_reset();
+
     do_cut_text(TRUE, FALSE, FALSE);
+
+    /* If the mark was set, blow away the cutbuffer on the next copy. */
+    next_contiguous_line = (mark_set ? NULL : openfile->current);
 }
 
 /* Cut from the current cursor position to the end of the file. */
-void do_cut_till_end(void)
+void do_cut_till_eof(void)
 {
     add_undo(CUT_EOF);
     do_cut_text(FALSE, TRUE, FALSE);
