@@ -1,4 +1,4 @@
-/* $Id: proto.h 4919 2014-05-28 13:24:05Z bens $ */
+/* $Id: proto.h 5052 2014-07-02 19:12:38Z bens $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -30,9 +30,11 @@
 #ifndef NANO_TINY
 extern sigjmp_buf jump_buf;
 extern bool jump_buf_main;
-extern bool use_undo;
 #endif
 
+extern bool meta_key;
+extern bool func_key;
+
 #ifndef DISABLE_WRAPJUSTIFY
 extern ssize_t fill;
 extern ssize_t wrap_at;
@@ -63,7 +65,6 @@ extern char *matchbrackets;
 #if !defined(NANO_TINY) && !defined(DISABLE_NANORC)
 extern char *whitespace;
 extern int whitespace_len[2];
-extern undo_type last_action;
 #endif
 
 extern const char *exit_tag;
@@ -81,7 +82,8 @@ extern char *quoteerr;
 #else
 extern size_t quotelen;
 #endif
-#endif
+#endif /* !DISABLE_JUSTIFY */
+
 extern bool nodelay_mode;
 extern char *answer;
 
@@ -101,20 +103,20 @@ extern char *full_operating_dir;
 extern char *alt_speller;
 #endif
 
-extern sc *sclist;
-extern subnfunc *allfuncs;
-extern subnfunc *exitfunc;
-extern subnfunc *uncutfunc;
 #ifndef DISABLE_COLOR
 extern syntaxtype *syntaxes;
 extern char *syntaxstr;
 #endif
 
 extern bool edit_refresh_needed;
-extern const shortcut *currshortcut;
-extern int currmenu;
 
-#ifndef NANO_TINY
+extern int currmenu;
+extern sc *sclist;
+extern subnfunc *allfuncs;
+extern subnfunc *exitfunc;
+extern subnfunc *uncutfunc;
+
+#ifndef DISABLE_HISTORIES
 extern filestruct *search_history;
 extern filestruct *searchage;
 extern filestruct *searchbot;
@@ -122,7 +124,6 @@ extern filestruct *replace_history;
 extern filestruct *replaceage;
 extern filestruct *replacebot;
 extern poshiststruct *poshistory;
-void update_poshistory(char *filename, ssize_t lineno, ssize_t xpos);
 #endif
 
 #ifdef HAVE_REGEX_H
@@ -138,12 +139,14 @@ extern color_pair interface_color_pair[NUMBER_OF_ELEMENTS];
 
 extern char *homedir;
 
+typedef void (*functionptrtype)(void);
+
 /* All functions in browser.c. */
 #ifndef DISABLE_BROWSER
 char *do_browser(char *path, DIR *dir);
 char *do_browse_from(const char *inpath);
 void browser_init(const char *path, DIR *dir);
-void parse_browser_input(int *kbinput, bool *meta_key);
+functionptrtype parse_browser_input(int *kbinput);
 void browser_refresh(void);
 bool browser_select_filename(const char *needle);
 int filesearch_init(void);
@@ -249,6 +252,7 @@ void color_update(void);
 
 /* All functions in cut.c. */
 void cutbuffer_reset(void);
+bool keeping_cutbuffer(void);
 void cut_line(void);
 #ifndef NANO_TINY
 void cut_marked(void);
@@ -257,7 +261,7 @@ void cut_to_eof(void);
 #endif
 void do_cut_text(
 #ifndef NANO_TINY
-	bool copy_text, bool cut_till_end, bool undoing
+	bool copy_text, bool cut_till_eof, bool undoing
 #else
 	void
 #endif
@@ -265,7 +269,7 @@ void do_cut_text(
 void do_cut_text_void(void);
 #ifndef NANO_TINY
 void do_copy_text(void);
-void do_cut_till_end(void);
+void do_cut_till_eof(void);
 #endif
 void do_uncut_text(void);
 
@@ -333,7 +337,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	*lastwastab, void (*refresh_func)(void), bool *list);
 #endif
 const char *tail(const char *foo);
-#if !defined(NANO_TINY) && !defined(DISABLE_NANORC)
+#ifndef DISABLE_HISTORIES
 char *histfilename(void);
 void load_history(void);
 bool writehist(FILE *hist, filestruct *histhead);
@@ -341,16 +345,27 @@ void save_history(void);
 int check_dotnano(void);
 void load_poshistory(void);
 void save_poshistory(void);
+void update_poshistory(char *filename, ssize_t lineno, ssize_t xpos);
 int check_poshistory(const char *file, ssize_t *line, ssize_t *column);
 #endif
-#ifndef DISABLE_COLOR
-void set_lint_shortcuts(void);
-void set_spell_shortcuts(void);
-#endif
 
 /* Some functions in global.c. */
 size_t length_of_list(int menu);
+key_type strtokeytype(const char *str);
+const sc *first_sc_for(int menu, void (*func)(void));
+int sc_seq_or(void (*func)(void), int defaultval);
+functionptrtype func_from_key(int *kbinput);
+void assign_keyinfo(sc *s);
+void print_sclist(void);
 void shortcut_init(void);
+#ifndef DISABLE_COLOR
+void set_lint_shortcuts(void);
+void set_spell_shortcuts(void);
+#endif
+const subnfunc *sctofunc(sc *s);
+const char *flagtostr(int flag);
+sc *strtosc(char *input);
+int strtomenu(char *input);
 #ifdef DEBUG
 void thanks_for_all_the_fish(void);
 #endif
@@ -359,7 +374,7 @@ void thanks_for_all_the_fish(void);
 #ifndef DISABLE_HELP
 void do_help(void (*refresh_func)(void));
 void help_init(void);
-void parse_help_input(int *kbinput, bool *meta_key);
+functionptrtype parse_help_input(int *kbinput);
 size_t help_line_len(const char *ptr);
 #endif
 void do_help_void(void);
@@ -453,7 +468,7 @@ void print_opt_full(const char *shortflag
 	, const char *desc);
 void usage(void);
 void version(void);
-int no_more_space(void);
+int more_space(void);
 int no_help(void);
 void nano_disabled_msg(void);
 void do_exit(void);
@@ -477,16 +492,15 @@ void enable_signals(void);
 void disable_flow_control(void);
 void enable_flow_control(void);
 void terminal_init(void);
-int do_input(bool *meta_key, bool *func_key, bool allow_funcs);
+int do_input(bool allow_funcs);
 #ifndef DISABLE_MOUSE
 int do_mouse(void);
 #endif
 void do_output(char *output, size_t output_len, bool allow_cntrls);
 
 /* All functions in prompt.c. */
-int do_statusbar_input(bool *meta_key, bool *func_key, bool *have_shortcut,
-	bool *ran_func, bool *finished, bool allow_funcs, void
-	(*refresh_func)(void));
+int do_statusbar_input(bool *ran_func, bool *finished,
+	void (*refresh_func)(void));
 #ifndef DISABLE_MOUSE
 int do_statusbar_mouse(void);
 #endif
@@ -504,38 +518,28 @@ bool do_statusbar_next_word(bool allow_punct);
 bool do_statusbar_prev_word(bool allow_punct);
 #endif
 void do_statusbar_verbatim_input(bool *got_enter);
-#ifndef NANO_TINY
-bool find_statusbar_bracket_match(bool reverse, const char
-	*bracket_set);
-void do_statusbar_find_bracket(void);
-#endif
 size_t statusbar_xplustabs(void);
 size_t get_statusbar_page_start(size_t start_col, size_t column);
 void reset_statusbar_cursor(void);
 void update_statusbar_line(const char *curranswer, size_t index);
 bool need_statusbar_horizontal_update(size_t pww_save);
 void total_statusbar_refresh(void (*refresh_func)(void));
-const sc *get_prompt_string(int *value, bool allow_tabs,
+functionptrtype get_prompt_string(int *value, bool allow_tabs,
 #ifndef DISABLE_TABCOMP
 	bool allow_files,
+	bool *list,
 #endif
 	const char *curranswer,
-	bool *meta_key, bool *func_key,
-#ifndef NANO_TINY
+#ifndef DISABLE_HISTORIES
 	filestruct **history_list,
 #endif
-	void (*refresh_func)(void), int menu
-#ifndef DISABLE_TABCOMP
-	, bool *list
-#endif
-	);
+	void (*refresh_func)(void));
 int do_prompt(bool allow_tabs,
 #ifndef DISABLE_TABCOMP
 	bool allow_files,
 #endif
 	int menu, const char *curranswer,
-	bool *meta_key, bool *func_key,
-#ifndef NANO_TINY
+#ifndef DISABLE_HISTORIES
 	filestruct **history_list,
 #endif
 	void (*refresh_func)(void), const char *msg, ...);
@@ -543,9 +547,11 @@ void do_prompt_abort(void);
 int do_yesno_prompt(bool all, const char *msg);
 
 /* Most functions in rcfile.c. */
+#if !defined(DISABLE_NANORC) || !defined(DISABLE_HISTORIES)
+char *parse_next_word(char *ptr);
+#endif
 #ifndef DISABLE_NANORC
 void rcfile_error(const char *msg, ...);
-char *parse_next_word(char *ptr);
 char *parse_argument(char *ptr);
 #ifndef DISABLE_COLOR
 char *parse_next_regex(char *ptr);
@@ -564,7 +570,7 @@ void parse_rcfile(FILE *rcstream
 #endif
 	);
 void do_rcfile(void);
-#endif
+#endif /* !DISABLE_NANORC */
 
 /* All functions in search.c. */
 #ifdef HAVE_REGEX_H
@@ -608,9 +614,12 @@ void goto_line_posx(ssize_t line, size_t pos_x);
 #ifndef NANO_TINY
 bool find_bracket_match(bool reverse, const char *bracket_set);
 void do_find_bracket(void);
-#ifndef DISABLE_NANORC
-bool history_has_changed(void);
+#ifndef DISABLE_TABCOMP
+char *get_history_completion(filestruct **h, const char *s, size_t len);
 #endif
+#endif
+#ifndef DISABLE_HISTORIES
+bool history_has_changed(void);
 void history_init(void);
 void history_reset(const filestruct *h);
 filestruct *find_history(const filestruct *h_start, const filestruct
@@ -620,9 +629,6 @@ char *get_history_older(filestruct **h);
 char *get_history_newer(filestruct **h);
 void get_history_older_void(void);
 void get_history_newer_void(void);
-#ifndef DISABLE_TABCOMP
-char *get_history_completion(filestruct **h, const char *s, size_t len);
-#endif
 #endif
 
 /* All functions in text.c. */
@@ -647,7 +653,7 @@ bool execute_command(const char *command);
 #endif
 #ifndef DISABLE_WRAPPING
 void wrap_reset(void);
-bool do_wrap(filestruct *line, bool undoing);
+bool do_wrap(filestruct *line);
 #endif
 #if !defined(DISABLE_HELP) || !defined(DISABLE_WRAPJUSTIFY)
 ssize_t break_line(const char *line, ssize_t goal
@@ -744,10 +750,10 @@ void dump_filestruct_reverse(void);
 void get_key_buffer(WINDOW *win);
 size_t get_key_buffer_len(void);
 void unget_input(int *input, size_t input_len);
-void unget_kbinput(int kbinput, bool meta_key, bool func_key);
+void unget_kbinput(int kbinput, bool metakey, bool funckey);
 int *get_input(WINDOW *win, size_t input_len);
-int get_kbinput(WINDOW *win, bool *meta_key, bool *func_key);
-int parse_kbinput(WINDOW *win, bool *meta_key, bool *func_key);
+int get_kbinput(WINDOW *win);
+int parse_kbinput(WINDOW *win);
 int get_escape_seq_kbinput(const int *seq, size_t seq_len);
 int get_escape_seq_abcd(int kbinput);
 int parse_escape_seq_kbinput(WINDOW *win, int kbinput);
@@ -763,8 +769,7 @@ int *parse_verbatim_kbinput(WINDOW *win, size_t *kbinput_len);
 #ifndef DISABLE_MOUSE
 int get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts);
 #endif
-const sc *get_shortcut(int menu, int *kbinput, bool *meta_key);
-const sc *first_sc_for(int menu, void (*func)(void));
+const sc *get_shortcut(int *kbinput);
 void blank_line(WINDOW *win, int y, int x, int n);
 void blank_titlebar(void);
 void blank_topbar(void);
@@ -795,27 +800,11 @@ void display_main_list(void);
 void do_cursorpos(bool constant);
 void do_cursorpos_void(void);
 void do_replace_highlight(bool highlight, const char *word);
-const char *flagtostr(int flag);
-const subnfunc *sctofunc(sc *s);
-const subnfunc *getfuncfromkey(WINDOW *win);
-void print_sclist(void);
-sc *strtosc(char *input);
-function_type strtokeytype(const char *str);
-int strtomenu(char *input);
-void assign_keyinfo(sc *s);
 void xon_complaint(void);
 void xoff_complaint(void);
-int sc_seq_or (void (*func)(void), int defaultval);
 void do_suspend_void(void);
-
-
 void enable_nodelay(void);
 void disable_nodelay(void);
-
-#ifdef HAVE_REGEX_H
-extern const char *regexp_msg;
-#endif
-
 #ifndef DISABLE_EXTRA
 void do_credits(void);
 #endif
@@ -836,7 +825,7 @@ void backup_file_void(void);
 void new_buffer_void(void);
 void backwards_void(void);
 void goto_dir_void(void);
-void no_replace_void(void);
-void ext_cmd_void(void);
+void flip_replace_void(void);
+void flip_execute_void(void);
 
 #endif /* !PROTO_H */
