@@ -1,4 +1,4 @@
-/* $Id: text.c 5055 2014-07-02 20:52:27Z bens $ */
+/* $Id: text.c 5068 2014-07-16 08:53:16Z bens $ */
 /**************************************************************************
  *   text.c                                                               *
  *                                                                        *
@@ -433,7 +433,7 @@ void do_undo(void)
     undo *u = openfile->current_undo;
     filestruct *t = 0;
     size_t len = 0;
-    char *undidmsg, *data;
+    char *data, *undidmsg = NULL;
 
     if (!u) {
 	statusbar(_("Nothing in undo buffer!"));
@@ -455,8 +455,8 @@ void do_undo(void)
     case ADD:
 	undidmsg = _("text add");
 	len = strlen(f->data) - strlen(u->strdata) + 1;
-        data = charalloc(len);
-        strncpy(data, f->data, u->begin);
+	data = charalloc(len);
+	strncpy(data, f->data, u->begin);
 	strcpy(&data[u->begin], &f->data[u->begin + strlen(u->strdata)]);
 	free(f->data);
 	f->data = data;
@@ -476,7 +476,6 @@ void do_undo(void)
 	break;
 #ifndef DISABLE_WRAPPING
     case SPLIT_END:
-	undidmsg = _("line wrap");
 	goto_line_posx(u->lineno, u->begin);
 	openfile->current_undo = openfile->current_undo->next;
 	openfile->last_action = OTHER;
@@ -484,6 +483,8 @@ void do_undo(void)
 	    do_undo();
 	u = openfile->current_undo;
 	f = openfile->current;
+    case SPLIT_BEGIN:
+	undidmsg = _("text add");
 	break;
 #endif /* !DISABLE_WRAPPING */
     case JOIN:
@@ -502,11 +503,13 @@ void do_undo(void)
     case CUT_EOF:
     case CUT:
 	undidmsg = _("text cut");
-        undo_cut(u);
+	undo_cut(u);
+	f = fsfromline(u->lineno);
 	break;
     case PASTE:
 	undidmsg = _("text uncut");
 	undo_paste(u);
+	f = fsfromline(u->lineno);
 	break;
     case ENTER:
 	undidmsg = _("line break");
@@ -554,7 +557,8 @@ void do_undo(void)
 	break;
     }
 
-    statusbar(_("Undid action (%s)"), undidmsg);
+    if (undidmsg)
+	statusbar(_("Undid action (%s)"), undidmsg);
 
     renumber(f);
     openfile->current_undo = openfile->current_undo->next;
@@ -568,7 +572,7 @@ void do_redo(void)
 {
     undo *u = openfile->undotop;
     size_t len = 0;
-    char *redidmsg, *data;
+    char *data, *redidmsg = NULL;
 
     for (; u != NULL && u->next != openfile->current_undo; u = u->next)
 	;
@@ -595,7 +599,7 @@ void do_redo(void)
     case ADD:
 	redidmsg = _("text add");
 	len = strlen(f->data) + strlen(u->strdata) + 1;
-        data = charalloc(len);
+	data = charalloc(len);
 	strncpy(data, f->data, u->begin);
 	strcpy(&data[u->begin], u->strdata);
 	strcpy(&data[u->begin + strlen(u->strdata)], &f->data[u->begin]);
@@ -608,7 +612,7 @@ void do_redo(void)
 	redidmsg = _("text delete");
 	len = strlen(f->data) + strlen(u->strdata) + 1;
 	data = charalloc(len);
-        strncpy(data, f->data, u->begin);
+	strncpy(data, f->data, u->begin);
 	strcpy(&data[u->begin], &f->data[u->begin + strlen(u->strdata)]);
 	free(f->data);
 	f->data = data;
@@ -622,7 +626,6 @@ void do_redo(void)
 	break;
 #ifndef DISABLE_WRAPPING
     case SPLIT_BEGIN:
-	redidmsg = _("line wrap");
 	goto_line_posx(u->lineno, u->begin);
 	openfile->current_undo = u;
 	openfile->last_action = OTHER;
@@ -630,6 +633,8 @@ void do_redo(void)
 	    do_redo();
 	u = openfile->current_undo;
 	goto_line_posx(u->lineno, u->begin);
+    case SPLIT_END:
+	redidmsg = _("text add");
 	break;
 #endif /* !DISABLE_WRAPPING */
     case JOIN:
@@ -675,7 +680,8 @@ void do_redo(void)
 	break;
     }
 
-    statusbar(_("Redid action (%s)"), redidmsg);
+    if (redidmsg)
+	statusbar(_("Redid action (%s)"), redidmsg);
 
     openfile->current_undo = u;
     openfile->last_action = OTHER;
@@ -996,7 +1002,7 @@ void update_undo(undo_type action)
     openfilestruct *fs = openfile;
 
 #ifdef DEBUG
-        fprintf(stderr, "action = %d, fs->last_action = %d, openfile->current->lineno = %ld",
+fprintf(stderr, "action = %d, fs->last_action = %d, openfile->current->lineno = %ld",
 		action, fs->last_action, (long)openfile->current->lineno);
 	if (fs->current_undo)
 	    fprintf(stderr, "fs->current_undo->lineno = %ld\n", (long)fs->current_undo->lineno);
@@ -1009,7 +1015,7 @@ void update_undo(undo_type action)
     if (action != fs->last_action
 	|| (action != ENTER && action != CUT && action != INSERT
 	    && openfile->current->lineno != fs->current_undo->lineno)) {
-        add_undo(action);
+	add_undo(action);
 	return;
     }
 
@@ -1019,7 +1025,7 @@ void update_undo(undo_type action)
     switch (u->type) {
     case ADD: {
 #ifdef DEBUG
-        fprintf(stderr, "fs->current->data = \"%s\", current_x = %lu, u->begin = %lu\n",
+	fprintf(stderr, "fs->current->data = \"%s\", current_x = %lu, u->begin = %lu\n",
 			fs->current->data, (unsigned long)fs->current_x, (unsigned long)u->begin);
 #endif
 	char *char_buf = charalloc(mb_cur_max());
@@ -1329,10 +1335,10 @@ ssize_t break_line(const char *line, ssize_t goal
 
 #ifndef DISABLE_HELP
     if (newln && blank_loc <= 0) {
-       /* If no blank was found, or was found only as the first
-        * character, force a line break. */
-       cur_loc -= char_len;
-       return cur_loc;
+	/* If no blank was found, or was found only as the first
+	 * character, force a line break. */
+	cur_loc -= char_len;
+	return cur_loc;
     }
 #endif
 
@@ -2856,7 +2862,7 @@ void do_spell(void)
     const char *spell_msg;
 
     if (ISSET(RESTRICTED)) {
-        nano_disabled_msg();
+	nano_disabled_msg();
 	return;
     }
 
@@ -2937,13 +2943,12 @@ void do_linter(void)
     }
 
     if (ISSET(RESTRICTED)) {
-        nano_disabled_msg();
+	nano_disabled_msg();
 	return;
     }
 
     if (openfile->modified) {
-	int i = do_yesno_prompt(FALSE,
-                _("Save modified buffer before linting?"));
+	int i = do_yesno_prompt(FALSE, _("Save modified buffer before linting?"));
 	if (i == -1) {
 	    statusbar(_("Cancelled"));
 	    lint_cleanup();
@@ -2989,13 +2994,13 @@ void do_linter(void)
 	/* Child continues (i.e. future spell process). */
 	close(lint_fd[0]);
 
-        /* Send spell's standard output/err to the pipe. */
-        if (dup2(lint_fd[1], STDOUT_FILENO) != STDOUT_FILENO)
-            exit(1);
-        if (dup2(lint_fd[1], STDERR_FILENO) != STDERR_FILENO)
-            exit(1);
+	/* Send spell's standard output/err to the pipe. */
+	if (dup2(lint_fd[1], STDOUT_FILENO) != STDOUT_FILENO)
+	    exit(1);
+	if (dup2(lint_fd[1], STDERR_FILENO) != STDERR_FILENO)
+	    exit(1);
 
-        close(lint_fd[1]);
+	close(lint_fd[1]);
 
 	/* Start the linter program; we are using $PATH. */
 	execvp(lintargs[0], lintargs);
@@ -3193,23 +3198,23 @@ void do_linter(void)
 
 	if (!s)
 	    continue;
-        else if (s->scfunc == do_cancel)
+	else if (s->scfunc == do_cancel)
 	    break;
-        else if (s->scfunc == do_help_void) {
+	else if (s->scfunc == do_help_void) {
 	    tmplint = NULL;
 	    do_help_void();
-        } else if (s->scfunc == do_page_down) {
+	} else if (s->scfunc == do_page_down) {
 	    if (curlint->next != NULL)
-	        curlint = curlint->next;
+		curlint = curlint->next;
 	    else {
-	        statusbar(_("At last message"));
+		statusbar(_("At last message"));
 		continue;
 	    }
 	} else if (s->scfunc == do_page_up) {
 	    if (curlint->prev != NULL)
 		curlint = curlint->prev;
 	    else {
-	        statusbar(_("At first message"));
+		statusbar(_("At first message"));
 		continue;
 	    }
 	}
